"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const chalk_1 = require("chalk");
const md_1 = __importDefault(require("../../md"));
const index_1 = __importDefault(require("./index"));
const imsize_1 = __importDefault(require("../imsize"));
const BLOCK_START = '{% changelog %}';
const BLOCK_END = '{% endchangelog %}\n';
function parseChangelogs(str, path) {
    const { parse, compile, env } = (0, md_1.default)({
        plugins: [index_1.default, imsize_1.default],
        extractChangelogs: true,
        path,
    });
    compile(parse(str));
    return env.changelogs || [];
}
const collect = (input, { path: filepath, log, changelogs, extractChangelogs }) => {
    let result = input;
    let lastPos = 0;
    const rawChanges = [];
    // eslint-disable-next-line no-constant-condition
    while (true) {
        const pos = result.indexOf(BLOCK_START, lastPos);
        lastPos = pos;
        if (pos === -1) {
            break;
        }
        const endPos = result.indexOf(BLOCK_END, pos + BLOCK_START.length);
        if (endPos === -1) {
            log.error(`Changelog block must be closed${filepath ? ` in ${(0, chalk_1.bold)(filepath)}` : ''}`);
            break;
        }
        const change = result.slice(pos, endPos + BLOCK_END.length);
        rawChanges.push(change);
        result = result.slice(0, pos) + result.slice(endPos + BLOCK_END.length);
    }
    if (rawChanges.length && changelogs && extractChangelogs) {
        const parsedChangelogs = parseChangelogs(rawChanges.join('\n\n'), filepath);
        if (parsedChangelogs.length !== rawChanges.length) {
            log.error(`Parsed cahngelogs less than expected${filepath ? ` in ${(0, chalk_1.bold)(filepath)}` : ''}`);
        }
        changelogs.push(...parsedChangelogs);
    }
    return result;
};
module.exports = collect;
//# sourceMappingURL=collect.js.map