"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getLogLevel = exports.errorToString = void 0;
const path_1 = require("path");
function errorToString(path, error, sourceMap) {
    const ruleMoniker = error.ruleNames
        ? error.ruleNames.join(path_1.sep)
        : // @ts-expect-error bad markdownlint typings
            error.ruleName + path_1.sep + error.ruleAlias;
    const lineNumber = sourceMap ? sourceMap[error.lineNumber] : error.lineNumber;
    return (`${path}${lineNumber ? `: ${lineNumber}:` : ':'} ${ruleMoniker} ${error.ruleDescription}` +
        (error.errorDetail ? ` [${error.errorDetail}]` : '') +
        (error.errorContext ? ` [Context: "${error.errorContext}"]` : ''));
}
exports.errorToString = errorToString;
function getLogLevel(opts) {
    const { ruleNames, logLevelsConfig, defaultLevel } = opts;
    const ruleName = ruleNames.filter((ruleName) => ruleName in logLevelsConfig)[0];
    return logLevelsConfig[ruleName] || defaultLevel;
}
exports.getLogLevel = getLogLevel;
//# sourceMappingURL=utils.js.map